import boto3
import sure  # noqa # pylint: disable=unused-import
import json
from botocore.exceptions import ClientError
import pytest
from moto import mock_appsync

schema = """type Mutation {
    putPost(id: ID!, title: String!): Post
}

"My custom post type"
type Post {
    id: ID!
    title: String!
}

type Query {
    singlePost(id: ID!): Post
}

schema {
    query: Query
    mutation: Mutation

}"""

schema_with_directives = """type Mutation {
    putPost(id: ID!, title: String!): Post
}

"My custom post type"
type Post {
    id: ID!
    title: String!
    createdAt: AWSDateTime!
}

type Query {
    singlePost(id: ID!): Post
}

schema {
    query: Query
    mutation: Mutation
    subscription: Subscription
}

type Subscription {
    onPostCreated(id: ID!): Post @aws_subscribe(mutations: ["putPost"])
}

"""


@mock_appsync
def test_start_schema_creation():
    client = boto3.client("appsync", region_name="us-east-2")
    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    resp = client.start_schema_creation(apiId=api_id, definition=b"sth")

    resp.should.have.key("status").equals("PROCESSING")


@mock_appsync
def test_get_schema_creation_status():
    client = boto3.client("appsync", region_name="eu-west-1")
    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(apiId=api_id, definition=schema.encode("utf-8"))
    resp = client.get_schema_creation_status(apiId=api_id)

    resp.should.have.key("status").equals("SUCCESS")
    resp.shouldnt.have.key("details")


@mock_appsync
def test_get_schema_creation_status_invalid():
    client = boto3.client("appsync", region_name="eu-west-1")
    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(apiId=api_id, definition=b"sth")
    resp = client.get_schema_creation_status(apiId=api_id)

    resp.should.have.key("status").equals("FAILED")
    resp.should.have.key("details").match("Syntax Error")


@mock_appsync
def test_get_type_from_schema():
    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(apiId=api_id, definition=schema.encode("utf-8"))
    resp = client.get_type(apiId=api_id, typeName="Post", format="SDL")

    resp.should.have.key("type")
    graphql_type = resp["type"]
    graphql_type.should.have.key("name").equals("Post")
    graphql_type.should.have.key("description").equals("My custom post type")
    graphql_type.should.have.key("arn").equals("arn:aws:appsync:graphql_type/Post")
    graphql_type.should.have.key("definition").equals("NotYetImplemented")
    graphql_type.should.have.key("format").equals("SDL")

    query_type = client.get_type(apiId=api_id, typeName="Query", format="SDL")["type"]
    query_type.should.have.key("name").equals("Query")
    query_type.shouldnt.have.key("description")


@mock_appsync
def test_get_introspection_schema_raise_gql_schema_error_if_no_schema():
    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    with pytest.raises(ClientError) as exc:
        client.get_introspection_schema(apiId=api_id, format="SDL")
    err = exc.value.response["Error"]
    err["Code"].should.equal("GraphQLSchemaException")
    # AWS API appears to return InvalidSyntaxError if no schema exists
    err["Message"].should.equal("InvalidSyntaxError")


@mock_appsync
def test_get_introspection_schema_sdl():
    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(apiId=api_id, definition=schema.encode("utf-8"))

    resp = client.get_introspection_schema(apiId=api_id, format="SDL")
    schema_sdl = resp["schema"].read().decode("utf-8")
    schema_sdl.should.contain("putPost(")
    schema_sdl.should.contain("singlePost(id: ID!): Post")


@mock_appsync
def test_get_introspection_schema_json():

    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(apiId=api_id, definition=schema.encode("utf-8"))

    resp = client.get_introspection_schema(apiId=api_id, format="JSON")
    schema_json = json.loads(resp["schema"].read().decode("utf-8"))
    schema_json.should.have.key("__schema")
    schema_json["__schema"].should.have.key("queryType")
    schema_json["__schema"].should.have.key("mutationType")
    schema_json["__schema"].should.have.key("subscriptionType")
    schema_json["__schema"].should.have.key("types")
    schema_json["__schema"].should.have.key("directives")


@mock_appsync
def test_get_introspection_schema_bad_format():
    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(apiId=api_id, definition=schema.encode("utf-8"))

    with pytest.raises(ClientError) as exc:
        client.get_introspection_schema(apiId=api_id, format="NotAFormat")
    err = exc.value.response["Error"]

    err["Code"].should.equal("BadRequestException")
    err["Message"].should.equal("Invalid format NotAFormat given")


@mock_appsync
def test_get_introspection_schema_include_directives_true():
    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(
        apiId=api_id, definition=schema_with_directives.encode("utf-8")
    )

    resp = client.get_introspection_schema(
        apiId=api_id, format="SDL", includeDirectives=True
    )

    schema_sdl = resp["schema"].read().decode("utf-8")

    schema_sdl.should.contain("@aws_subscribe")


@mock_appsync
def test_get_introspection_schema_include_directives_false():
    client = boto3.client("appsync", region_name="us-east-2")

    api_id = client.create_graphql_api(name="api1", authenticationType="API_KEY")[
        "graphqlApi"
    ]["apiId"]

    client.start_schema_creation(
        apiId=api_id, definition=schema_with_directives.encode("utf-8")
    )

    resp = client.get_introspection_schema(
        apiId=api_id, format="SDL", includeDirectives=False
    )

    schema_sdl = resp["schema"].read().decode("utf-8")

    schema_sdl.shouldnt.contain("@aws_subscribe")



import boto3
import sure  # noqa # pylint: disable=unused-import
import pytest
import botocore.exceptions
from moto import mock_apigatewayv2


@mock_apigatewayv2
def test_create_domain_name():
    client = boto3.client("apigatewayv2", region_name="us-east-1")
    domain_name = "dev"
    tags = {"tag": "it"}
    expected_keys = [
        "DomainName",
        "ApiMappingSelectionExpression",
        "DomainNameConfigurations",
        "MutualTlsAuthentication",
        "Tags",
    ]

    post_resp = client.create_domain_name(DomainName=domain_name, Tags=tags)
    get_resp = client.get_domain_name(DomainName=domain_name)

    # check post response has all keys
    for key in expected_keys:
        post_resp.should.have.key(key)

    # check get response has all keys
    for key in expected_keys:
        get_resp.should.have.key(key)

    # check that values are equal for post and get of same resource
    for key in expected_keys:
        post_resp.get(key).should.equal(get_resp.get(key))

    # ensure known values are set correct in post response
    post_resp.get("DomainName").should.equal(domain_name)
    post_resp.get("Tags").should.equal(tags)


@mock_apigatewayv2
def test_create_domain_name_already_exists():
    client = boto3.client("apigatewayv2", region_name="us-east-1")
    client.create_domain_name(DomainName="exists.io")

    with pytest.raises(botocore.exceptions.ClientError) as exc:
        client.create_domain_name(DomainName="exists.io")

    err = exc.value.response["Error"]
    err["Code"].should.equal("ConflictException")
    err["Message"].should.equal("The domain name resource already exists.")


@mock_apigatewayv2
def test_get_domain_names():
    client = boto3.client("apigatewayv2", region_name="us-east-1")
    dev_domain = client.create_domain_name(DomainName="dev.service.io")
    prod_domain = client.create_domain_name(DomainName="prod.service.io")

    # sanity check responses
    dev_domain.should.have.key("DomainName").equals("dev.service.io")
    prod_domain.should.have.key("DomainName").equals("prod.service.io")

    # make comparable
    del dev_domain["ResponseMetadata"]
    del prod_domain["ResponseMetadata"]

    get_resp = client.get_domain_names()
    get_resp.should.have.key("Items")
    get_resp.get("Items").should.contain(dev_domain)
    get_resp.get("Items").should.contain(prod_domain)


@mock_apigatewayv2
def test_delete_domain_name():
    client = boto3.client("apigatewayv2", region_name="ap-southeast-1")
    post_resp = client.create_domain_name(DomainName="dev.service.io")
    client.delete_domain_name(DomainName="dev.service.io")
    get_resp = client.get_domain_names()

    del post_resp["ResponseMetadata"]
    get_resp.should.have.key("Items")
    get_resp.get("Items").should_not.contain(post_resp)


@mock_apigatewayv2
def test_delete_domain_name_dne():
    client = boto3.client("apigatewayv2", region_name="ap-southeast-1")
    with pytest.raises(botocore.exceptions.ClientError) as exc:
        client.delete_domain_name(DomainName="dne.io")

    err = exc.value.response["Error"]
    err["Code"].should.equal("NotFoundException")
    err["Message"].should.equal(
        "The domain name resource specified in the request was not found."
    )



"""Unit tests for identitystore-supported APIs."""
from uuid import UUID

import boto3
import sure  # noqa # pylint: disable=unused-import

from moto import mock_identitystore


# See our Development Tips on writing tests for hints on how to write good tests:
# http://docs.getmoto.org/en/latest/docs/contributing/development_tips/tests.html


@mock_identitystore
def test_create_group():
    client = boto3.client("identitystore", region_name="ap-southeast-1")
    identity_store_id = "d-9067028cf5"
    create_resp = client.create_group(
        IdentityStoreId=identity_store_id,
        DisplayName="test_group",
        Description="description",
    )
    assert create_resp["IdentityStoreId"] == identity_store_id
    assert UUID(create_resp["GroupId"])



import boto3
import sure  # noqa # pylint: disable=unused-import
import pytest

from moto import mock_dynamodb
from botocore.exceptions import ClientError


def _create_user_table():
    client = boto3.client("dynamodb", region_name="us-east-1")
    client.create_table(
        TableName="users",
        KeySchema=[{"AttributeName": "username", "KeyType": "HASH"}],
        AttributeDefinitions=[{"AttributeName": "username", "AttributeType": "S"}],
        ProvisionedThroughput={"ReadCapacityUnits": 5, "WriteCapacityUnits": 5},
    )
    client.put_item(
        TableName="users", Item={"username": {"S": "user1"}, "foo": {"S": "bar"}}
    )
    client.put_item(
        TableName="users", Item={"username": {"S": "user2"}, "foo": {"S": "bar"}}
    )
    client.put_item(
        TableName="users", Item={"username": {"S": "user3"}, "foo": {"S": "bar"}}
    )
    return client


@mock_dynamodb
def test_batch_items_returns_all():
    dynamodb = _create_user_table()
    returned_items = dynamodb.batch_get_item(
        RequestItems={
            "users": {
                "Keys": [
                    {"username": {"S": "user0"}},
                    {"username": {"S": "user1"}},
                    {"username": {"S": "user2"}},
                    {"username": {"S": "user3"}},
                ],
                "ConsistentRead": True,
            }
        }
    )["Responses"]["users"]
    assert len(returned_items) == 3
    assert [item["username"]["S"] for item in returned_items] == [
        "user1",
        "user2",
        "user3",
    ]


@mock_dynamodb
def test_batch_items_throws_exception_when_requesting_100_items_for_single_table():
    dynamodb = _create_user_table()
    with pytest.raises(ClientError) as ex:
        dynamodb.batch_get_item(
            RequestItems={
                "users": {
                    "Keys": [
                        {"username": {"S": "user" + str(i)}} for i in range(0, 104)
                    ],
                    "ConsistentRead": True,
                }
            }
        )
    ex.value.response["Error"]["Code"].should.equal("ValidationException")
    msg = ex.value.response["Error"]["Message"]
    msg.should.contain("1 validation error detected: Value")
    msg.should.contain(
        "at 'requestItems.users.member.keys' failed to satisfy constraint: Member must have length less than or equal to 100"
    )


@mock_dynamodb
def test_batch_items_throws_exception_when_requesting_100_items_across_all_tables():
    dynamodb = _create_user_table()
    with pytest.raises(ClientError) as ex:
        dynamodb.batch_get_item(
            RequestItems={
                "users": {
                    "Keys": [
                        {"username": {"S": "user" + str(i)}} for i in range(0, 75)
                    ],
                    "ConsistentRead": True,
                },
                "users2": {
                    "Keys": [
                        {"username": {"S": "user" + str(i)}} for i in range(0, 75)
                    ],
                    "ConsistentRead": True,
                },
            }
        )
    ex.value.response["Error"]["Code"].should.equal("ValidationException")
    ex.value.response["Error"]["Message"].should.equal(
        "Too many items requested for the BatchGetItem call"
    )


@mock_dynamodb
def test_batch_items_with_basic_projection_expression():
    dynamodb = _create_user_table()
    returned_items = dynamodb.batch_get_item(
        RequestItems={
            "users": {
                "Keys": [
                    {"username": {"S": "user0"}},
                    {"username": {"S": "user1"}},
                    {"username": {"S": "user2"}},
                    {"username": {"S": "user3"}},
                ],
                "ConsistentRead": True,
                "ProjectionExpression": "username",
            }
        }
    )["Responses"]["users"]

    returned_items.should.have.length_of(3)
    [item["username"]["S"] for item in returned_items].should.be.equal(
        ["user1", "user2", "user3"]
    )
    [item.get("foo") for item in returned_items].should.be.equal([None, None, None])

    # The projection expression should not remove data from storage
    returned_items = dynamodb.batch_get_item(
        RequestItems={
            "users": {
                "Keys": [
                    {"username": {"S": "user0"}},
                    {"username": {"S": "user1"}},
                    {"username": {"S": "user2"}},
                    {"username": {"S": "user3"}},
                ],
                "ConsistentRead": True,
            }
        }
    )["Responses"]["users"]

    [item["username"]["S"] for item in returned_items].should.be.equal(
        ["user1", "user2", "user3"]
    )
    [item["foo"]["S"] for item in returned_items].should.be.equal(["bar", "bar", "bar"])


@mock_dynamodb
def test_batch_items_with_basic_projection_expression_and_attr_expression_names():
    dynamodb = _create_user_table()
    returned_items = dynamodb.batch_get_item(
        RequestItems={
            "users": {
                "Keys": [
                    {"username": {"S": "user0"}},
                    {"username": {"S": "user1"}},
                    {"username": {"S": "user2"}},
                    {"username": {"S": "user3"}},
                ],
                "ConsistentRead": True,
                "ProjectionExpression": "#rl",
                "ExpressionAttributeNames": {"#rl": "username"},
            }
        }
    )["Responses"]["users"]

    returned_items.should.have.length_of(3)
    [item["username"]["S"] for item in returned_items].should.be.equal(
        ["user1", "user2", "user3"]
    )
    [item.get("foo") for item in returned_items].should.be.equal([None, None, None])


@mock_dynamodb
def test_batch_items_should_throw_exception_for_duplicate_request():
    client = _create_user_table()
    with pytest.raises(ClientError) as ex:
        client.batch_get_item(
            RequestItems={
                "users": {
                    "Keys": [
                        {"username": {"S": "user0"}},
                        {"username": {"S": "user0"}},
                    ],
                    "ConsistentRead": True,
                }
            }
        )
    ex.value.response["Error"]["Code"].should.equal("ValidationException")
    ex.value.response["Error"]["Message"].should.equal(
        "Provided list of item keys contains duplicates"
    )


@mock_dynamodb
def test_batch_items_should_return_16mb_max():
    """
    A single operation can retrieve up to 16 MB of data [...]. BatchGetItem returns a partial result if the response size limit is exceeded [..].

    For example, if you ask to retrieve 100 items, but each individual item is 300 KB in size,
    the system returns 52 items (so as not to exceed the 16 MB limit).

    It also returns an appropriate UnprocessedKeys value so you can get the next page of results.
    If desired, your application can include its own logic to assemble the pages of results into one dataset.
    """
    client = _create_user_table()
    # Fill table with all the data
    for i in range(100):
        client.put_item(
            TableName="users",
            Item={"username": {"S": f"largedata{i}"}, "foo": {"S": "x" * 300000}},
        )

    resp = client.batch_get_item(
        RequestItems={
            "users": {
                "Keys": [{"username": {"S": f"largedata{i}"}} for i in range(75)],
                "ConsistentRead": True,
            }
        }
    )

    resp["Responses"]["users"].should.have.length_of(55)
    unprocessed_keys = resp["UnprocessedKeys"]["users"]["Keys"]
    # 75 requested, 55 returned --> 20 unprocessed
    unprocessed_keys.should.have.length_of(20)

    # Keys 55-75 are unprocessed
    unprocessed_keys.should.contain({"username": {"S": "largedata55"}})
    unprocessed_keys.should.contain({"username": {"S": "largedata65"}})

    # Keys 0-55 are processed in the regular response, so they shouldn't show up here
    unprocessed_keys.shouldnt.contain({"username": {"S": "largedata45"}})



import boto3

from moto import mock_batch
from tests import DEFAULT_ACCOUNT_ID


@mock_batch
def test_create_scheduling_policy():
    client = boto3.client("batch", "us-east-2")
    resp = client.create_scheduling_policy(name="test")
    resp.should.have.key("name").equals("test")
    resp.should.have.key("arn").equals(
        f"arn:aws:batch:us-east-2:{DEFAULT_ACCOUNT_ID}:scheduling-policy/test"
    )


@mock_batch
def test_describe_default_scheduling_policy():
    client = boto3.client("batch", "us-east-2")
    arn = client.create_scheduling_policy(name="test")["arn"]

    resp = client.describe_scheduling_policies(arns=[arn])
    resp.should.have.key("schedulingPolicies").length_of(1)

    policy = resp["schedulingPolicies"][0]
    policy["arn"].should.equal(arn)
    policy["fairsharePolicy"].should.equal(
        {"computeReservation": 0, "shareDecaySeconds": 0, "shareDistribution": []}
    )
    policy["tags"].should.equal({})


@mock_batch
def test_describe_scheduling_policy():
    client = boto3.client("batch", "us-east-2")
    arn = client.create_scheduling_policy(
        name="test",
        fairsharePolicy={
            "shareDecaySeconds": 1,
            "computeReservation": 2,
            "shareDistribution": [{"shareIdentifier": "A", "weightFactor": 0.1}],
        },
    )["arn"]

    resp = client.list_scheduling_policies()
    resp.should.have.key("schedulingPolicies")
    arns = [a["arn"] for a in resp["schedulingPolicies"]]
    arns.should.contain(arn)

    resp = client.describe_scheduling_policies(arns=[arn])
    resp.should.have.key("schedulingPolicies").length_of(1)

    policy = resp["schedulingPolicies"][0]
    policy["arn"].should.equal(arn)
    policy["fairsharePolicy"].should.equal(
        {
            "computeReservation": 2,
            "shareDecaySeconds": 1,
            "shareDistribution": [{"shareIdentifier": "A", "weightFactor": 0.1}],
        }
    )
    policy["tags"].should.equal({})


@mock_batch
def test_delete_scheduling_policy():
    client = boto3.client("batch", "us-east-2")
    arn = client.create_scheduling_policy(name="test")["arn"]

    client.delete_scheduling_policy(arn=arn)

    resp = client.describe_scheduling_policies(arns=[arn])
    resp.should.have.key("schedulingPolicies").length_of(0)


@mock_batch
def test_update_scheduling_policy():
    client = boto3.client("batch", "us-east-2")
    arn = client.create_scheduling_policy(name="test")["arn"]

    client.update_scheduling_policy(
        arn=arn,
        fairsharePolicy={
            "computeReservation": 5,
            "shareDecaySeconds": 10,
            "shareDistribution": [],
        },
    )

    resp = client.describe_scheduling_policies(arns=[arn])
    resp.should.have.key("schedulingPolicies").length_of(1)

    policy = resp["schedulingPolicies"][0]
    policy["arn"].should.equal(arn)
    policy["fairsharePolicy"].should.equal(
        {"computeReservation": 5, "shareDecaySeconds": 10, "shareDistribution": []}
    )



"""Handles incoming identitystore requests, invokes methods, returns responses."""
import json

from moto.core.responses import BaseResponse
from .models import identitystore_backends, IdentityStoreBackend


class IdentityStoreResponse(BaseResponse):
    """Handler for IdentityStore requests and responses."""

    def __init__(self) -> None:
        super().__init__(service_name="identitystore")

    @property
    def identitystore_backend(self) -> IdentityStoreBackend:
        """Return backend instance specific for this region."""
        return identitystore_backends[self.current_account][self.region]

    def create_group(self) -> str:
        identity_store_id = self._get_param("IdentityStoreId")
        display_name = self._get_param("DisplayName")
        description = self._get_param("Description")
        group_id, identity_store_id = self.identitystore_backend.create_group(
            identity_store_id=identity_store_id,
            display_name=display_name,
            description=description,
        )
        return json.dumps(dict(GroupId=group_id, IdentityStoreId=identity_store_id))



"""Test different server responses."""
import sure  # noqa # pylint: disable=unused-import

import json
import moto.server as server
from moto import mock_textract


@mock_textract
def test_textract_start_text_detection():
    backend = server.create_backend_app("textract")
    test_client = backend.test_client()

    headers = {"X-Amz-Target": "X-Amz-Target=Textract.StartDocumentTextDetection"}
    request_body = {
        "DocumentLocation": {
            "S3Object": {"Bucket": "bucket", "Name": "name", "Version": "version"}
        }
    }

    resp = test_client.post("/", headers=headers, json=request_body)
    data = json.loads(resp.data.decode("utf-8"))
    resp.status_code.should.equal(200)
    data["JobId"].should.be.an(str)


@mock_textract
def test_textract_start_text_detection_without_document_location():
    backend = server.create_backend_app("textract")
    test_client = backend.test_client()

    headers = {"X-Amz-Target": "X-Amz-Target=Textract.StartDocumentTextDetection"}
    resp = test_client.post("/", headers=headers, json={})
    data = json.loads(resp.data.decode("utf-8"))
    resp.status_code.should.equal(400)
    data["__type"].should.equal("InvalidParameterException")
    data["message"].should.equal(
        "An input parameter violated a constraint. For example, in synchronous operations, an InvalidParameterException exception occurs when neither of the S3Object or Bytes values are supplied in the Document request parameter. Validate your parameter before calling the API operation again."
    )


@mock_textract
def test_textract_get_text_detection():
    backend = server.create_backend_app("textract")
    test_client = backend.test_client()

    headers = {"X-Amz-Target": "X-Amz-Target=Textract.StartDocumentTextDetection"}
    request_body = {
        "DocumentLocation": {
            "S3Object": {"Bucket": "bucket", "Name": "name", "Version": "version"}
        }
    }

    resp = test_client.post("/", headers=headers, json=request_body)
    start_job_data = json.loads(resp.data.decode("utf-8"))

    headers = {"X-Amz-Target": "X-Amz-Target=Textract.GetDocumentTextDetection"}
    request_body = {
        "JobId": start_job_data["JobId"],
    }
    resp = test_client.post("/", headers=headers, json=request_body)
    resp.status_code.should.equal(200)
    data = json.loads(resp.data.decode("utf-8"))
    data["JobStatus"].should.equal("SUCCEEDED")


@mock_textract
def test_textract_get_text_detection_without_job_id():
    backend = server.create_backend_app("textract")
    test_client = backend.test_client()

    headers = {"X-Amz-Target": "X-Amz-Target=Textract.GetDocumentTextDetection"}
    request_body = {
        "JobId": "invalid_job_id",
    }
    resp = test_client.post("/", headers=headers, json=request_body)
    resp.status_code.should.equal(400)
    data = json.loads(resp.data.decode("utf-8"))
    data["__type"].should.equal("InvalidJobIdException")
    data["message"].should.equal("An invalid job identifier was passed.")
